% The SMoSU algorithm for endmember selection
%
% Input:
% image data and spectral library
% Output:
% Indexes of the selected endmembers
%
% Please cite the following papers when using this code:
% [1] Xu X, Shi Z, Pan B. 
% L0-based sparse hyperspectral unmixing using spectral information and a multi-objectives formulation. 
% ISPRS Journal of Photogrammetry and Remote Sensing, 2018, 141: 46-58.
% [2] X. Fu, W.-K. Ma, J. M. Bioucas-Dias, and T.-H. Chan,
% Semiblind Hyperspectral Unmixing in the Presence of Spectral Library Mismatches,
% IEEE Transactions on Geoscience and Remote Sensing, 2016, 54(9): 5171-5184.
% [3] Zhang Q, Li H. 
% MOEA/D: A multiobjective evolutionary algorithm based on decomposition. 
% IEEE Transactions on evolutionary computation, 2007, 11(6): 712-731.
%
% Code by Xia Xu, e-mail: xuxia@buaa.edu.cn

clear
clc
% Input data, including image data and the library
load data_example
global img_params lambda pop
% Regularization parameter for spectral information
lambda = 0.2;
% Prune the spectral library
[w,Rw]=estNoise(Y,'off');
[kf,Ek,E,delta_p]=hysime(Y,w,Rw,'off');
E = E(:,1:kf);
Pyort = (eye(size(A,1))-E*E');
[ind_dect] = RMUSIC_dict_prune(A,0,Pyort,40);
A_MUSIC = A(:,ind_dect);%Library after pruning

img_params = struct('img',Y,'lib',A_MUSIC,'k', kf, 'img_size',size(Y,2), 'lib_size', size(A_MUSIC,2));
clearvars -except img_params ind_dect
% Define objectives
mop = objectives(img_params);
% Population size
pop = 100;
% Neiborhood size
nic = 10;
% Iteration times
iter = 100;
% After 90% iterations lambda is directly set as 0
iter_lambda = 0.9*iter;
% Main function. pp is the objectives values, and individual stores the
% selected endmembers.
[ pp, individual] = smosu(mop, ind_dect, img_params, ...
    'popsize', pop, 'niche', nic, 'iteration', iter , 'iter_lambda', iter_lambda , 'method', 'te');

